import numpy as np
from src.plot import plot_xy
import time
from src.TiseSolverFgh1D import TiseSolverFgh1D
import os
from pathlib import Path

if __name__ == "__main__":
    # - Input parameters
    # -- Position
    xa = -10.0  # Position x = [xa, xb]
    xb = 10.0
    nx = 2**6  # Number of x points

    # -- Output folder
    output_folder_path = "./out/"

    # -- Preparetion
    folder_path = Path(output_folder_path)  # Make folder
    folder_path.mkdir(exist_ok=True, parents=True)

    # =====================
    #         TISE
    # =====================

    # -- Solve TISE using Fourier-Grid Hamiltonian method
    solver = TiseSolverFgh1D(xa, xb, nx)
    x, Vx = solver.abscissa_potential()
    start_time = time.time()  # Record the start time
    eigenvalues, eigenfunctions = solver.solve()
    end_time = time.time()  # Record the end time
    print(f"  Elapsed time TISE: {end_time - start_time:.6f} seconds")

    # -- Output TISE results
    x_dict = {}
    y_dict = {}
    x_dict["V(x)"] = np.array(x)
    y_dict["V(x)"] = np.array(Vx)
    for n in range(5):
        x_dict[f"Idx {n:>2d}"] = np.array(x)
        y_dict[f"Idx {n:>2d}"] = eigenfunctions[:, n].real + eigenvalues[n]
    output_image_name = os.path.join(output_folder_path, "eigenfunctions.png")
    plot_xy(
        x_dict,
        y_dict,
        show_gui=False,
        output_image_name=output_image_name,
    )

    # - Eigenvalues output
    output_csv_name = os.path.join(output_folder_path, "tise_eigenvalues.csv")
    with open(output_csv_name, "w") as f:
        for idx, value in enumerate(eigenvalues):
            f.write(f"{idx},{value}\n")
